unit ClTable;
(*
 Clone Table V1.00 - released July 09, 1997

 Disclaimer:
 This is a Freeware component. Use it at your own risk.
 You can send any bug, suggestion and 'thanks a lot' to demian@bhnet.com.br
 New versions can be found at: http://www.bhnet.com.br/~demian
                               http://web.unix.horizontes.com.br/~demian

 If you do any modification to this code, please send me a copy.
 If you use this code on any 'real-life' application, please let me know.
 If you think this code is completely useless, please DON'T LET ME KNOW!

 What it does:
 Provides the client a relatively quick way to get a cursor for a table that is
 already opened. This is accomplished through the new method 'AssignCursor'.
 The cloned cursor can be used as a regular cursor, inheriting certain
 properties from the source cursor, but remaining completely independent in terms
 of position and ordering.  The cloned cursor inherits the following properties
 from the source cursor: Current index
                         Range
                         Translate mode
                         Share mode
	                 Position
	                 Field maps
	                 Filters
 How to use it:
 1. Install the component [VNM Pallete];
 2. Drop one TTable and one CloneTable onto a form;
 3. Set the TTable properties (DatabaseName, TableName, etc);
 4. 'Clone' table one: CloneTable1.AssignCursor(Table1);
                       CloneTable1.Open;

 Notice:
 1. I've designed a TCloneTable descendent for allowing null values, but not
    duplicated ones, in secondary key fields not indexed with the 'Unique'
    clause. EMail me if you're interested;
 2. Make sure the source table is opened before calling 'AssignCursor';
 3. Because the cloned table is just a cursor, not all source table
    properties are inherited (see list in 'What it does').
 4. Since the AssignCursor is a virtual method, you can override it for 'cloning'
    other table properties before opening the cursor. Or, if you don't like the
    objects oriented standards, you can set the properties right here, in this
    unit AssignCursor method, like this:  self.DatabaseName := Source.DatabaseName
                                          self.TableName := Source.TableName}
 5. As a bonus, I've added a write method to the Handle property, so now you can
    access, in a normal way, those 'misterious' BDE handles. Try the example below
    for displaying a list of available BDE drivers (use a TDatasource/TDBGrid):
    {Notice: add 'DBIProcs' to the unit Uses clause}
    var
      MyHandle: HDBICur;
    begin
      DbiOpenDriverList(MyHandle);
      CloneTable1.Handle := MyHandle;
      CloneTable1.Open;
    end;
*)

interface

uses
  Classes,SysUtils,DB,DBTables,Forms,DBIProcs,DBITypes,DBIErrs,WinTypes;

type
  TCloneTable = class(TTable)
  private
    FHandle: HDBICur;
  protected
    function CreateHandle: HDBICur; override;
  public
    Property Handle: HDBICur read FHandle write FHandle;
    procedure AssignCursor (const Source: TTable); virtual;
end;

procedure Register;

implementation

{$IFDEF WIN32}
 {$R *.R32}
{$ELSE}
 {$R *.R16}
{$ENDIF}

{______________________________________________________________________________}
procedure Register;
begin
  RegisterComponents('VNM',[TCloneTable]);
end;

{______________________________________________________________________________}
function TCloneTable.CreateHandle: HDBICur;
{Notice: this function is called whenever the 'Open' method is executed}
begin
  if FHandle <> nil then
    result := FHandle                                             {the fast way}
  else
    result := inherited CreateHandle;                             {the slow way}
end;

{______________________________________________________________________________}
procedure TCloneTable.AssignCursor (const Source: TTable);
begin
  Active := false;                                 {target table must be closed}
  Check(DBICloneCursor(Source.Handle,false,false,FHandle));
end;

end.
